/*
    Kommando Menu Routinen

    Dieses Modul enthält Routinen, welche aus dem Auswahlmenü
    aufgerufen werden, nachdem der Kommandostring ausgewertet
    wurde.

    (C) Christian Julius 11.2014
    christian.julius@gmx.de

*/
#include <asm/z80/features.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include "main.h"
#include "mcurses.h"
#include "hexedit.h"
#include "driver.h"
#include "command.h"
#include "tools.h"
#include "sd.h"

#pragma disable_warning 196         // Const Zeiger verloren ????

extern __at (ROM_START) unsigned char memory[0xffff];         // Kompletter Speicher
extern struct system_type sys;             // System Infos
extern struct cl_type cl;                   // Eingabepuffer mit allen Parametern

///////////////////////////////////////////////////////////
// Zerlegt den Kommandostring in Befehl und Parameter
// Befehl in cmd[]
// Parameter in param[]
// Eingabe: Call by reference struct auf cl_type
// Rückgabe: Anzahl Parameter


/*
struct cl_type {
       char inbuf[CMDBUFSIZE];              // Eingabepuffer
       char cmd[CMD_LEN];                   // Kommandostring
       char param[MAX_PARAMS][PARAM_LEN];   // Parameterliste (Strings)
       char no_params;                      // Anzahl übergebener Parameter
};
*/

// Cursor auf 1 Zeichen aktueller Zeile zurücksetzen
void mvpos1()
{
    uint8_t ypos,xpos;
    getyx(ypos,xpos);
    move(ypos,0);
}

// Erzeugt ein CR/LF
void nextline()
{
    uint8_t ypos,xpos;

    getyx(ypos,xpos);
    move(ypos+1,0);
}

// Koordinaten x,y unten links
void gotoxy(uint8_t x, uint8_t y)
{
    if ((x <= LINES) &  (y<= COLS))
        move(y,LINES-x);
}


// Statuszeile unten ausgeben
/*
void statusline(char* zeile)
{
    #define STATYPOS    23
    char xpos,ypos;

    getyx(ypos,xpos);           // Alte Pos speichern
    move(STATYPOS,0);          // Statuszeile
    deleteln();
    addstr("Z80 > ");
    addstr(zeile);
    move(ypos,xpos);
}
*/

// Gibt Text aus und wartet auf <Return>
void press_return()
{
    addstr(" Press <RETURN>");
    while(keypress()!=0x0d);
}


void hexdump()
{
    uint16_t adres;

// Eingabeprüfung
    if (cl.no_params>=1) {
        adres = hex2int(cl.param[0]);
        hexedit(adres);
    }
    else
        hexedit(0);

}


// Gibt Informationen über den Systemzustand aus
char menu_system_info()
{
    uint8_t buf[8];
    uint16_t check;

    nextline();
    if (sys.f_prog_loaded) {
        addstr(" A File was loaded!");
        nextline();
        xtoa(buf,sys.user_start);
        addstr(" Start :  0x"); addstr(buf); nextline();
        xtoa(buf,sys.user_end);
        addstr(" End   :  0x"); addstr(buf); nextline();
        addstr(" Size  :  ");   addint(sys.user_length);
        addstr(" Bytes");       nextline();
        check = checksum(sys.user_start,sys.user_end);
        xtoa(buf,check);
        addstr(" CRC   :  0x"); addstr(buf);
        xtoa(buf,sys.user_check);
        addstr(" / 0x"); addstr(buf);

        // Prüfe Checksumme des Programmes
        if (check == sys.user_check)
            addstr(" --> OK!");
        else {
            addstr(" --> ERROR! Erasing....");
            memset((uint16_t*)RAM_START,0,DATA_START-RAM_START-1);
            sys.f_prog_loaded = 0;
        }

    }
    else
        addstr(" No File was loaded!");

    press_return();

    return(1);
}

// FIXXXXME!
void menu_getad()
{
    uint8_t foo = 0;
    uint8_t old = 0;

     addstr("\r\nZ80 > Reading ADC... (CR to stop)");
     curs_set(0);
     while (keypress()!=0x0d)
     {
         foo = adc_read();
         if (foo != old) {
            old = foo;
            move(menu_entries+4,6);
            addstr("ADC:      ");
            move(menu_entries+4,12);
            addint((uint8_t)adc_read());
            delay(3000);
            }
      }
      curs_set(2);
}

// Lädt ein ausführbares Programm von der Uart
// oder SD Karte herunter
char menu_load_file()
{
    uint8_t xpos,ypos;

    memset((uint16_t*)RAM_START,0,DATA_START-RAM_START-1);
    addstr("\r\nZ80 > Waiting for PC Data!");
    getyx(ypos,xpos);
    move(ypos+2,0);
    if (ihx_decode()) {
        sys.f_prog_loaded = 1;
        addstr("\r\nIHX8 data loaded.");
        return(1);
    }
    else {
        sys.f_prog_loaded = 0;
        addstr("\r\nIHX8 CRC / Range Error!");
        return (0);
    }

}

//////////////////////////////////////////////////////////
// Speichert ein Programm oder Daten auf SD Karte
char menu_save_file()
{
    uint8_t descr[40];
    uint8_t i;

    // Eingabeprüfung
    if (cl.no_params<1) {
        addstr("\r\nZ80 > Filename missing!");
        press_return();
        return (0);
    }

    if (strlen(cl.param[0])>11) {
        addstr("\r\nZ80 > Filename too long (11)!");
        press_return();
        return (0);
    }

    // Aus Parameter 1-n evtl.Beschreibung zusammen bauen
    // in descr[40]
    memset(descr,0,sizeof(descr));
    if (cl.no_params > 1)
    {
        for (i=1;i<cl.no_params;i++)
        {
            if (strlen(descr)<MAX_DESCR_SIZE) {
                strcat(descr,cl.param[i]);
                strcat(descr," ");
            }
        }
    }
    else
        strcpy(descr,"<none>");

    addstr("\r\nZ80 > Saving File to SD...");
    if (!sd_save_prog(cl.param[0],descr))
        addstr("Error!");
    else
        addstr("Success!");
    //press_return();
    return (1);


}

//////////////////////////////////////////
// Startet ein Programm im RAM
//////////////////////////////////////////

char  menu_run_file()
{

    if (sys.f_prog_loaded)
    {
        addstr("\n\rZ80 > Wait...");
        // Teste die Prüfsumme des Programms
         if (checksum(sys.user_start,sys.user_end) != sys.user_check) {
            // CRC ausgeben
            addstr("\r\nZ80 > CRC Error!");
            press_return();
            return (0);
        }

        // Starte ins neue Programm hinein
        flush_tx();
        __asm
            di
            ld  sp,#0x0000   ;; Stack an Spitze des RAM legen
            jp RAM_START
        __endasm;
     }
     else  {
         addstr("\r\nZ80 > No File loaded!");
         press_return();
     }

     return (0);
}

void menu_del_file()
{
   press_return();
}

//////////////////////////////////////////
// Listet die SD karte auf
//////////////////////////////////////////

void menu_cat_sd()
{
    press_return();
}


// Setzt einen Port auf einen Wert
void menu_setport()
{
   press_return();
}

//////////////////////////////////////////
// Prüft das RAM auf Fehler
//////////////////////////////////////////

void menu_test_ram()
{

    uint8_t pattern;
    uint16_t round,count;

    round   = 0;
    nextline();
    addstr("Z80 > Infinite RAM Test ... <Reset>");
    curs_set(0);
    pattern = 0x55;

    while (1) {
        pattern = ~pattern;
        LedOut(pattern);
        move(menu_entries+3,0);
        addstr("Z80 > Test: "); addint(round);

        // Einschreiben
        memset((void*)RAM_START,pattern,DATA_START-RAM_START);
        // Vergleiche...
        for (count=RAM_START;count<DATA_START;count++)
        {
            if (memory[count]!=pattern) {
                addstr(" ERROR! : "); addint(pattern);
                SetDigit(0x0e);
                flush_tx();
                stop_cpu();
             }
        }
        addstr("  >RAM OK<!");
        round++;
    }


}

