;--------------------------------------------------------------------------
; crt0.s - Generic crt0.s for a Z80
;
; Minimum Z80 System mit Mostek 3801 Multi I/O Chip
;
;  (C)Christian Julius 10.2014
;  christian.julius@gmx.de
;--------------------------------------------------------------------------

    .module crt0            ; Modul Name fr den Linker

    ; Memory Map Adressen der Zeropage (vom Benutzer definierbar)
    ; Definitions

    offset           .equ   0x0000   ; Basisadresse des Codes
    reset_vector     .equ   offset
    stack            .equ   0x0000   ; Stack
    adr_vec_table    .equ   0x40     ; Mode 2 Vektor Tabelle fr STI Mostek Multi I/O
    adr_nmi          .equ   0x66     ;  NMI Reset Vektor
    adr_copyright    .equ   0x80
    ram_prog         .equ   0x4000   ; Beginn des Userprogrammes im RAM
    heap_size        .equ   1024     ; Benutzer definierte Heap Groesse

    ; Globale Funktionen und Variablen fr das C-Programm

    .globl  _main
    .globl  _nmi_vint       ; Funktion des NMI Handlers
    .globl __sdcc_heap_init
    .globl _letzte_code_adresse
    .globl _erste_code_adresse
    .globl _nmi_reset_vector

    ; Interruptroutinen fr den Mostek MK3801 Multi I/O Baustein
    ; Manual STI Baustein MK3801 Figure 7
    .globl  _int_sti_gpi_0  ; General Purpose Interrupt 0
    .globl  _int_sti_gpi_1  ; General Purpose Interrupt 1
    .globl  _int_sti_gpi_2  ; General Purpose Interrupt 2
    .globl  _int_sti_gpi_3  ; General Purpose Interrupt 3
    .globl  _int_sti_timer_d
    .globl  _int_sti_timer_c
    .globl  _int_sti_gpi_4  ; General Purpose Interrupt 4
    .globl  _int_sti_gpi_5  ; General Purpose Interrupt 5
    .globl  _int_sti_timer_b
    .globl  _int_sti_transmit_error
    .globl  _int_sti_transmit_buffer_empty
    .globl  _int_sti_receive_error
    .globl  _int_sti_receive_buffer_full
    .globl  _int_sti_timer_a
    .globl  _int_sti_gpi_6  ; General Purpose Interrupt 6
    .globl  _int_sti_gpi_7  ; General Purpose Interrupt 7

    .globl l__INITIALIZER
    .globl s__INITIALIZER
    .globl s__INITIALIZED

    ; -------------------------------------------------------------

    .area   _HEADER (ABS)

    ; Reset vector bei Kaltstart und Sprung ins User Programm
    .org    reset_vector
    _erste_code_adresse:
     jp init

    ; Warmstart mit NMI Interrupt (ohne Speicher loeschen)
     .org    reset_vector+3
    _nmi_reset_vector:
     jp  init2

    ; Tabelle der RST Vektoren fr Software Interrupts
     .org 0x08
     jp ram_prog+0x0008
     .org 0x10
     jp ram_prog+0x0010
     .org 0x18
     jp ram_prog+0x0018
     .org 0x20
     jp ram_prog+0x0020
     .org 0x28
     jp ram_prog+0x0028
     .org 0x30
     jp ram_prog+0x0030
     .org 0x38
     jp ram_prog+0x0038

    ;///////////////////////////////////////////////////////////
    ; Tabelle der Mode 2 Int Vektoren auf die Handler Funktionen im C Modul
    ; Reihenfolge beachten !!! Siehe Manual Mostek 3801, Interrupt Tabelle

    .org adr_vec_table

    .dw (_int_sti_gpi_0)
    .dw (_int_sti_gpi_1)
    .dw (_int_sti_gpi_2)
    .dw (_int_sti_gpi_3)
    .dw (_int_sti_timer_d)
    .dw (_int_sti_timer_c)
    .dw (_int_sti_gpi_4)
    .dw (_int_sti_gpi_5)
    .dw (_int_sti_timer_b)
    .dw (_int_sti_transmit_error)
    .dw (_int_sti_transmit_buffer_empty)
    .dw (_int_sti_receive_error)
    .dw (_int_sti_receive_buffer_full)
    .dw (_int_sti_timer_a)
    .dw (_int_sti_gpi_6)
    .dw (_int_sti_gpi_7)

    ;///////////////////////////////////////////////////////////
    ; NMI Interrupt 0x66, muss mit retn abgeschlossen werden
    .org  adr_nmi
    jp _nmi_vint        ; Aufruf Handler im C Modul main.c

    ; //////////////////////////////////////////////////////////
    .org adr_copyright
    .asciz "Z80/MK3801 System (C) 12.2014 by C.Julius"

    ;///////////////////////////////////////////////////////////
    ; Anordung der Segmente fuer den Linker, ab hier nur relative
    ; Adressen

      .area  _CODE
      .area  _INITIALIZER
      .area  _HOME
      .area  _GSINIT
      .area  _GSFINAL
      .area  _DATA
      .area  _INITIALIZED
      .area  _BSEG
      .area  _BSS
      .area  _HEAP

; ---  Defintion der Heap Groesse
__sdcc_heap_start::
      .ds heap_size

      .area _HEAP_END
__sdcc_heap_end::
      .ds 1

    ;///////////////////////////////////////////////////////////
    ;; ------- Start des Init Codes ab hier -----------

    .area _CODE
    init:

    ld  sp,#stack       ;; Stack an Spitze des RAM legen

    ; Sind wir grad im RAM oder EPROM? => PC ermitteln
    call l1
l1: pop af              ; (Highbyte) )PC holen
    cp a,#0x00          ; Zeropage = 0 ?
    jp Z, init2         ; =Z, wir sind im RAM, dann berspringe

    ; Wir sind im EPROM, dann pruefe, ob ein Programm im RAM liegt
    ld a, (ram_prog)   ; Lade Start des RAM 0x4000
    cp #0xC3           ; vergleiche mit "C3" = JP Befehl
    jp Z, ram_prog     ; Ja, dann springe direkt ins Ram hinein

    ; Sonst Ram ausnullen
    xor     a                       ; clear a and carry
    ld      bc,#(0xffff-ram_prog)   ; ram size left
    ld      hl,#ram_prog            ; starting from 4000
    ld      de,#ram_prog+1
    ld      (hl),a        ; 0 -> (4000)
    ldir                  ; (HL) -> (DE)

    ; Normaler Programmstart ab hier
init2:
    ld  sp,#stack   ;; Stack an Spitze des RAM legen
    call gsinit     ;; Static Variablen init
;   -------------------------------------------
    call  _main     ;; Hauptprogramm aufrufen
;   -------------------------------------------
    di              ;; Bei Verlassen INTs aus
endlos:             ;; Und Bremse rein
    halt
    jr endlos

;; -----Globale Initialisierungen (Code liegt hinter Code Segment)------------

   .area  _GSINIT

 gsinit:

    ; Initialisiere den HEAP
    call __sdcc_heap_init

    ; Initialisiere globale Variablen im RAM
    ld  bc, #l__INITIALIZER
    ld  a, b
    or  a, c
    jr  Z, weiter
    ld  de, #s__INITIALIZED
    ld  hl, #s__INITIALIZER
    ldir

weiter:

    ; // Hier muss ein ret rein
    .area _GSFINAL
    ret
    ;Markiert das Ende des Programmes
    .ascii "Z80"
    _letzte_code_adresse:





