/////////////////////////////////////////////////////////
/*
   Minimum Z80 System mit Mostek 3801 Multi I/O Chip
   (C)Christian Julius 10.2014
   christian.julius@gmx.de

   Interrupt Handler Routine Mode 2
   siehe crt0.s fr Vektoren

*/
/////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "main.h"
#include "driver.h"

extern struct system_type sys;             // System Infos

// RX und TX Buffer....
extern struct tx_type tx;
extern struct rx_type rx;

/*  Interrupt Zusatz:
  __interrupt erzeugt ein reti am Ende, Register werden gesichert
  __naked verhindert Erzeugung von Code zur Registersicherung auf dem Stack
   und erzeugt kein RETI
*/

////////////////////////////////////////////////////////
// NMI Handler siehe crt0.s Definition
// __critical __interrupt erzeugt ein retn am Ende statt reti

extern char nmi_reset_vector[];

void nmi_vint(void) __critical __interrupt
{
     __asm
         di
         jp _nmi_reset_vector
    __endasm ;

}

/////////////////////////////////////////////////////////
// Handler der Mode 2 Interrupts der STI
//
// ISR: Uart RX puffer (1 Byte) ist voll, es sind Zeichen da!
// getchar() muss diese abholen und Zeiger verstellen
// f_buf_overflow muss vom Hauptprogramm zurck gesetzt werden


void int_sti_receive_buffer_full(void)  __interrupt  // geprft = ok
{
    uint8_t zeichen;
    zeichen = STI_UDR;
    // Noch Platz im Puffer?

    if (rx.bytes_to_read < BUF_SIZE)
    {
        rx.buf[rx.wr_ptr] = zeichen;                // Zeichen in Ringpuffer schreiben
        rx.wr_ptr = (rx.wr_ptr +1) & rx.mask;       // Rotieren
        rx.f_buf_overflow = false;
        rx.bytes_to_read++;
    } else
        rx.f_buf_overflow = true;                   // Buffer Overflow

    EI;
}

////////////////////////////////////////////////
// ISR: Uart TX Sendepuffer ist leer
// putchar()fllt ihn wieder
//
// Vernderte globale Variablen:
// tx.bytes_to_send, tx.rd_ptr

void int_sti_transmit_buffer_empty(void)  __interrupt
{
    // Sind noch Zeichen im TX Buffer?
    if (tx.bytes_to_send > 0)  {                // ja.....
        STI_UDR   = tx.buf[tx.rd_ptr];          // Byte einschreiben
        tx.rd_ptr = (tx.rd_ptr +1) & tx.mask;   // Read Pointer rollen
        tx.f_idle = false;
        tx.bytes_to_send--;
    }
    else {
        // nein, letztes Zeichen ging grad raus
        tx.f_idle = true;
    }

    EI;

}


////////////////////////////////////////
// ------- Unbenutzte Ints -------------
////////////////////////////////////////

void int_sti_timer_a(void) __interrupt __interrupt __naked
{
    EI_RETI;
}


// AD Wandler Interrupt ber STI
void int_sti_gpi_0(void) __interrupt __naked
{
   EI_RETI;
}

// Interrupt des Timers B
void int_sti_timer_b(void)  __interrupt __naked
{
   EI_RETI;
}

// Interrupt des Timers C
void int_sti_timer_c(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_timer_d(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_gpi_1(void)  __interrupt __naked
{

    EI_RETI;
}
void int_sti_gpi_2(void)  __interrupt __naked
{
    EI_RETI;
}
void int_sti_gpi_3(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_gpi_4(void)  __interrupt __naked
{
   EI_RETI;
}

void int_sti_gpi_5(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_gpi_6(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_gpi_7(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_transmit_error(void)  __interrupt __naked
{
    EI_RETI;
}

void int_sti_receive_error(void)  __interrupt __naked
{
    EI_RETI;
}






