/////////////////////////////////////////////////////////
/*
   Minimum Z80 System mit Mostek 3801 Multi I/O Chip
   (C)Christian Julius 10.2014
   christian.julius@gmx.de

   Hauptprogramm

*/
/////////////////////////////////////////////////////////


// --- Standard Header ---
#include <asm/z80/features.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
// --- Projekt Header ---
#include "xprintf.h"
#include "command.h"
#include "mcurses.h"
#include "hexedit.h"
#include "driver.h"
#include "sd.h"
#include "tools.h"
#include "main.h"

//#pragma disable_warning 154
#pragma disable_warning 196         // Const Zeiger verloren ????

// --------Interne Variablen -------
// Definiert in crt0.s Nicht ändern!
extern char erste_code_adresse[];         // Zeiger auf SDCC Globals
extern char letzte_code_adresse[];

struct system_type sys;             // System Infos
struct cl_type cl;                  // Eingabepuffer mit allen Parametern
uint8_t AD_Value;                    // Aktueller AD Wert (von TA ausgelesen

////////////////////////////////////////////
//      Globale Variablen
/////////////////////////////////////////////

extern void life();

// Absolute Adressen des gesamten RAMs (ohne Stack) und ROMs
// Diese Array sind Overlays und benötigen keinen Speicherplatz
__at (ROM_START) unsigned char memory[0xffff];         // Kompletter Speicher

// Menu Ausgabemc

const char* const menu[] = {
  "Z80 Monitor V3.2 (C) C.Julius 12.2014\r\n",
  "\r\n[load] <sd n>.<uart>.........Load File <n> <source>"
  "\r\n[save] <n>......................... Save File to SD",
  "\r\n[del]  <n>................... Delete File n from SD",
  "\r\n[cat]................................... Catalog SD",
  "\r\n[run].................................. Run Program",
  "\r\n[info]..................................System Info",
  "\r\n[clear].................................. Clear RAM",
  "\r\n[dump]................................. Hex Monitor",
  "\r\n[ramtest]................................. Test RAM",
  "\r\n[adc]..................................... Read ADC",
  "\r\n[life].................................Game of Life",
  "\r\n[set] <port> <$value>....Set Port A/B/C to <$value>",
  "\r\n",
  "\r\nuser>"
};

void show_menu()
{
    uint8_t buf[8];
    char i;
    clear();
    // Menu anzeigen
    for (i=0;i<menu_entries;i++)
      addstr(menu[i]);

    // ROM Prüfsumme oben ausgeben  
    xtoa(buf,sys.rom_check);
    move(0,45);
    addstr("0x"),
    addstr(buf);
      
    move(menu_entries+1,6);
}
 


//#define DEBUG_COMMAND

// Rueckgabe 2 = String zu lang, 1 = ok, 0 = keine Zeichen
char split_command(struct cl_type* ptr)
{
    uint8_t  cbuf[CMDBUFSIZE];               // Kopie Eingabepuffer
    uint8_t* cptr;                          // Zeiger auf Eingabepuffer
    uint8_t  j;

    // Stringzerlegung
    strcpy(cbuf,ptr->inbuf);    // String Kopie erzeugen

    j = 0;
    cptr = strtok(cbuf," ");             // Zeichentrenner und erster Abschnitt
    strcpy(ptr->cmd,cptr);
    while(cptr != 0 && (j < MAX_PARAMS))
    {
        cptr = strtok(NULL," ");
        if (cptr != NULL) {
            if (strlen(cptr)<PARAM_LEN)
                strcpy(ptr->param[j++],cptr);
            else
                return (2);
        }
    }

    ptr->no_params = j;

    #ifdef DEBUG_COMMAND
    // Testweise Ausgabe der Zerlegung
    xprintf("\r\ncmd: %s",ptr->cmd);
    xprintf("\r\nParams: %d",j);
    for (i=0;i < ptr->no_params;i++)
        xprintf("\r\nParam %d : %s",i,ptr->param[i]);
    #endif

    // 1 = ok
    return (1);
}

////////////////////////////////////////////////////////////////////////
//                             M A I N
////////////////////////////////////////////////////////////////////////

void main()
{
    uint8_t res;
    uint8_t old = 0;

    CLI;
    // Sichere die Start- und Endadresse des Codes
    sys.code_start = (uint16_t)erste_code_adresse;
    sys.code_end   = (uint16_t)letzte_code_adresse;
    sys.code_length = letzte_code_adresse-erste_code_adresse;
    sys.rom_check  = checksum(ROM_START,ROM_END);
    
    // Ist ein Userprogramm von vor dem Reset vorhanden?
    // Alle Variableninhalte sind dann noch vorhanden! (Prüfsumme)
    sys.f_prog_loaded = (memory[RAM_START] == 0xC3) ? true:false;

    // Initialisierung
    InitHardware();                 // Hardware Init
    init_spi();                     // SPI vorbereiten
    init_buffer();                  // Buffer Init
    rx_full_enable(true);           // Uart RX Ints einschalten
    tx_full_enable(true);
    EI;

    initscr();                    // Bildschirm vorbereiten
    curs_set(2);
    attrset(A_NORMAL);

    xdev_out(putchar);
    
    while (1) {
        do {
            show_menu();
            // Wenn File geladen dieses anzeigen
            if (sys.f_prog_loaded)
                mvaddstr(23,0,"File in Memory!");
            else
                mvaddstr(23,0,"Memory cleared!");

            move(menu_entries+1,6);

            // Leerstrings ignorieren
            do {
                getnstr(cl.inbuf,CMDBUFSIZE-1);
            } while (strlen(cl.inbuf)==0);

            // String splitten
            res = split_command(&cl);
            // und Test auf Maximallaenge der Wörter
            if (res == 2) {
                addstr("\r\nZ80 > Only 12 chars per word!");
                press_return();
            }
        } while (res != 1);

        ////////////////////////////////////////////////////////
        // Hex Editor
        ////////////////////////////////////////////////////////
        if (strcmp(cl.cmd,"dump")==0)
        {
                hexdump();
        }
        ////////////////////////////////////////////////////////
        // Lade Userprogramm
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"load")==0)
        {
            if (menu_load_file())
               sys.f_prog_loaded = true;
            else
               sys.f_prog_loaded = false;

            press_return();
        }
        ////////////////////////////////////////////////////////
        // Speichere Userprogramm
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"save")==0) {
            if (sys.f_prog_loaded)
                menu_save_file();
            else
            addstr("\r\nZ80 > No File loaded!");
            press_return();
        }
        ////////////////////////////////////////////////////////
        // Loesche File von SD Karte
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"del")==0) {
            menu_del_file();
        }
        ////////////////////////////////////////////////////////
        // Katalog SD Karte
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"cat")==0) {
            menu_cat_sd();
        }
        ////////////////////////////////////////////////////////
        // Starte Userprogramm
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"run")==0) {
            menu_run_file();
         }
        ////////////////////////////////////////////////////////
        // System Infos anzeigen
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"info")==0) {
            menu_system_info();
         }
        ////////////////////////////////////////////////////////
        // Clear memory
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"clear") == 0)
        {
            addstr("\r\nZ80 > Erasing RAM...");
            memset((uint16_t*)RAM_START,0,DATA_START-RAM_START-1);
            addstr("\r\nZ80 > RAM erased!");
            sys.f_prog_loaded = 0;
            press_return();
        }
        ////////////////////////////////////////////////////////
        // Ad Wandler auslesen
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"adc") == 0)  {
            menu_getad();
        }
        ////////////////////////////////////////////////////////
        // Game of Life
        ////////////////////////////////////////////////////////
        else if (strcmp(cl.cmd,"life") == 0) {
            life();
        }
        ////////////////////////////////////////////////////////
        // RAM Dauertest
        ////////////////////////////////////////////////////////
        else  if (strcmp(cl.cmd,"ramtest") ==0 )
        {
            menu_test_ram();
        }

        else {
            addstr("\r\nZ80 > Unknown Command! ");
            press_return();
        }

    }

    //endwin ();

}



