﻿/*
    tools.c
    Nützliche Routinen, die oft benötigt werden.
*/
#include <asm/z80/features.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
#include "mcurses.h"
#include "tools.h"
#include "command.h"
#include "driver.h"
#include "main.h"

__at (ROM_START) uint8_t mem[0xffff];         // Kompletter Speicher
extern struct system_type sys;                      // System Infos

uint16_t checksum(uint16_t start, uint16_t end)
{
    uint16_t i, sum = 0;

    for (i=start;i<end;i++)
        sum += mem[i];

    return(sum);
}

// Konvertiere 4-stelligen Hex Ascii nach Integer
uint16_t hex2int(uint8_t* buf)
{
    uint8_t* p;
    uint8_t i;
    uint16_t off = 0;

    p = buf;
    for (i=0;*p; i++)
    {
        if (i > 0)
             off <<= 4;
        off += xtoi (*(p++));
    }

    return(off);
}

// Konvertiere Int -> Hex String
// Eingabe: Integer Zahl, Ausgabe: HEX-String
void xtoa(uint8_t *buf, uint16_t val)
{
  uint8_t *p;
  uint8_t *firstdig;
  uint8_t temp;
  uint8_t digval;

  // Speicher Pointer auf erstes Digit
  p = buf;
  firstdig = p;
  do
  {
    digval = val % 16;
    val /= 16;

    // In Ascii unwandeln
    if (digval > 9)
      *p++ = (digval - 10 + 'A');
    else
      *p++ = (digval + '0');
  } while (val > 0);

  // Reihenfolge umkehren
  *p-- = '\0';
  do
  {
    temp      = *p;
    *p        = *firstdig;
    *firstdig = temp;
    p--;
    firstdig++;
  } while (firstdig < p);
}

//////////////////////////////////////////////
//       Intel Hex Decodierung
//////////////////////////////////////////////

uint8_t csum;

// Konvertiere Hex Digit -> Char
uint8_t xtoi (uint8_t ch)
{
    uint8_t c = toupper(ch);
    c -= '0';
    if (c > 9)
        c -= ('A' - '0' - 10);

    return c;
}

// Hole 2 stelligen HEX Wert
static uint8_t gethex()
{
  uint8_t val;

  // Hi Byte zuerst im Stream
  val = (xtoi(getchar()) << 4) + xtoi(getchar());
  csum += val;

  return val;
}

#define VERBOSE

char ihx_decode()
{
    #define DATA_RECORD     0x00
    #define END_RECORD      0x01

   static uint8_t len,record;
   static uint16_t adress,max,min,target;
   static uint8_t ch,i,k,cnt;

   k   = 0;
   cnt = 0;
   max = 0;
   min = 0xffff;

   do {
        // warte auf Startzeichen
        do ch = getchar();
        while (ch != ':');

        csum = 0;
        len = gethex();                          // Hole Anzahl Zeichen des Satzes
        adress = (gethex() << 8) + gethex();     // Hole Zieladresse des Satzes
        record = gethex();                       // Hole Record Type

        if (record == DATA_RECORD)               // Record 0 = Daten
        {
            if (adress < RAM_START)              // Nicht ins ROM schreiben!
                return(0);

            for (i=0;i<len;i++)                  // Hole Datenbytes ab
            {
                target = adress+i;
                mem[target] = gethex();
                LedOut(mem[target]);
                if (target > max)                // Höchste Adresse merken
                    max = target;
                if (target < min)
                    min = target;
            }
            #ifdef VERBOSE
                addstr(".");
                if (++cnt>51) {
                    mvpos1();
                    clrtoeol();
                    cnt = 0;
                }
            #endif

            gethex();                            // Prüfsumme überprüfen
            if (csum != 0)
                return (0);
            SetDigit(k++);
        }
    } while (record != END_RECORD);

    // Programmparameter abspeichern
    sys.user_start = min;
    sys.user_end = max+1;
    sys.user_length = sys.user_end-sys.user_start;

    // Dateiende abwarten
    do
        ch = getchar();
    while (ch!=0x0a);

    // Prüfumme über das ganze Programm bilden
    sys.user_check = checksum(sys.user_start,sys.user_end);

    return (1);

    #undef DATA_RECORD
    #undef END_RECORD
}

/////////////////////////////////////////////
// Zufallszahlengenerator

#ifdef USE_SIMPLE_RANDOM

static uint16_t seed=10000;

void sprng(uint16_t start)
{
    seed = start;
}

uint16_t prng(uint16_t max)
{
    uint16_t result;

    seed ^= seed<<7;
    seed ^= seed>>9;
    seed ^= seed<<5;
    result = seed % max;

    return (result);
}

#endif

// Quicksort Algorithmus
#ifdef USE_QUICKSORT

static int split ( int a[ ], uint16_t lower, uint16_t upper )
{
    int i, t;
    uint16_t p,q;

    p = lower + 1 ;
    q = upper ;
    i = a[lower] ;

    while ( q >= p )  {
        while ( a[p] < i )
            p++ ;

        while ( a[q] > i )
            q-- ;

        if ( q > p ) {
            t = a[p] ;
            a[p] = a[q] ;
            a[q] = t ;
        }
    }

    t = a[lower] ;
    a[lower] = a[q] ;
    a[q] = t ;

    return q ;
}

void quicksort ( int a[ ], uint16_t lower, uint16_t upper )
{
    uint16_t i ;
    if ( upper > lower )
    {
        i = split ( a, lower, upper ) ;
        quicksort ( a, lower, i - 1 ) ;
        quicksort ( a, i + 1, upper ) ;
    }
}

#endif

// Mersenne - Twister Algorithmus

#ifdef USE_MERSENNE_TWISTER

uint32_t TT800(uint16_t range) {
#define N  25
#define M   7
   static const uint32_t A[2] = { 0, 0x8ebfd028 };
   static uint32_t y[N];
   static int index = N+1;
   uint32_t  e;

   if (index >= N) {
     int k;
     if (index > N) {
        unsigned r = 9, s = 3402;
        for (k=0; k<N; ++k) {
          r = 509845221 * r + 3;
          s *= s + 1;
          y[k] = s + (r >> 10);
        }
     }
     for (k=0; k<N-M; ++k)
        y[k] = y[k+M] ^ (y[k] >> 1) ^ A[y[k] & 1];
     for (; k<N; ++k)
        y[k] = y[k+(M-N)] ^ (y[k] >> 1) ^ A[y[k] & 1];
     index = 0;
   }

   e = y[index++];

   e ^= (e <<  7) & 0x2b5b2500;
   e ^= (e << 15) & 0xdb8b0000;
   e ^= (e >> 16);
   return (e % range);
#undef N
#undef M
}

#endif


